@@ -27,11 +27,9 @@ class ProblemSummary:
     # Main tests
     test_results_count: int = 0
     test_results_passing_count: int = 0
-    test_results_failing_count: int = 0
 
     # Weight
     test_results_passing_weight: Decimal = Decimal(0)
-    test_results_total_weight: Decimal = Decimal(0)
 
     def __post_init__(self):
         """Cache some common analysis of the data."""
@@ -39,9 +37,6 @@ class ProblemSummary:
         for task in self.problem.grader.tasks:
             result = self.report[task.name]
 
-            if not result.visible:
-                continue
-
             if task.stage == "setup":
                 if result.error is not None:
                     self.setup_results_errored.append(result)
@@ -50,12 +45,9 @@ class ProblemSummary:
                 self.test_results_count += 1
 
                 # Increment counts
-                self.test_results_total_weight += task.weight
                 if result.passing and result.complete:
                     self.test_results_passing_count += 1
                     self.test_results_passing_weight += task.weight
-                else:
-                    self.test_results_failing_count += 1
 
     @property
     def test_results_passing(self) -> Iterator[Result]:
@@ -78,23 +70,19 @@ class ProblemSummary:
                     yield result
 
     @property
-    def tests_fraction(self) -> str:
-        return f"{self.test_results_passing_count}/{self.test_results_passing_count + self.test_results_failing_count}"
-
-    @property
-    def points_ratio(self) -> Decimal:
-        return self.problem.grading.points_automated / self.test_results_total_weight
+    def passing_fraction(self) -> str:
+        return f"{self.test_results_passing_count}/{self.test_results_count}"
 
     @property
     def points_fraction(self) -> str:
         """Format a fraction."""
 
         numerator = self.test_results_passing_weight
-        denominator = self.test_results_total_weight
+        denominator = self.problem.grader.test.weight
         if denominator == 0:
             return f"0/0"
 
-        points_total_automated = self.problem.grading.points_automated
+        points_total_automated = self.problem.grading.automated.points
         if denominator != points_total_automated:
             numerator = numerator / denominator * points_total_automated
             denominator = points_total_automated
@@ -106,8 +94,12 @@ class ProblemSummary:
 
         if self.test_results_count == 0:
             return Decimal("0")
-        numerator = sum_weights(self.test_results_passing)
-        denominator = numerator + sum_weights(self.test_results_failing)
+
+        numerator = self.test_results_passing_weight
+        denominator = self.problem.grader.test.weight
+        if denominator == 0:
+            return Decimal("0")
+
         return Decimal(numerator) / Decimal(denominator)
 
 
@@ -136,13 +128,16 @@ def format_report_markdown(
         options: dict = None) -> str:
     """Return a formatted markdown report."""
 
+    assignment = GradingAssignment.read(grading_path)
+    with report_path.open() as file:
+        report = AssignmentReport.load(json.load(file), assignment)
+    if report.partial:
+        return "Cannot format a partial report!"
+
     if template_path is None:
         template_path = DEFAULT_TEMPLATE_PATH
 
     environment = jinja2_create_environment(custom_template_path=template_path)
-    assignment = GradingAssignment.read(grading_path)
-    with report_path.open() as file:
-        report = AssignmentReport.load(json.load(file), assignment)
     summary = summarize(assignment, report)
 
     environment.globals.update(assignment=assignment, summary=summary, options=options)
